#include "stdafx.h"


namespace bbs {

BBSManager::BBSManager()
{
}

BBSManager::~BBSManager()
{
}

bool BBSManager::init(const std::wstring& url, const std::wstring& shitaraba, const std::wstring& userAgent, int timeout)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.bbs);

	network::URLParser u(url);

	if (match(u.host(), shitaraba))
	{
		bbs_.reset(new ShitarabaBBS(url, userAgent, timeout));
	}
	else
	{
		bbs_.reset(new BaseBBS(url, userAgent, timeout));
	}

	boost::recursive_mutex::scoped_lock lock2(mutex_.value);
	info_.reset();
	subject_.reset();
	dat_.reset();
	pos_.reset();

	if (bbs_ && *bbs_)
	{
		info_.emplace();
		bbs_->getInfo(*info_);
		return true;
	}
	return false;
}

void BBSManager::uninit()
{
	boost::recursive_mutex::scoped_lock lock(mutex_.bbs);
	bbs_.reset();

	boost::recursive_mutex::scoped_lock lock2(mutex_.value);
	info_.reset();
	subject_.reset();
	dat_.reset();
	pos_.reset();
}

bool BBSManager::change(const std::wstring& key)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.bbs);
	if (bbs_ && *bbs_ && bbs_->change(key))
	{
		boost::recursive_mutex::scoped_lock lock(mutex_.value);
		dat_.reset();
		pos_.reset();
		info_.emplace();
		bbs_->getInfo(*info_);
		return true;
	}
	return false;
}

int BBSManager::loadSetting(const std::wstring& userAgent, int timeout)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.bbs);
	if (bbs_ && *bbs_)
	{
		int res = bbs_->loadSetting(userAgent, timeout);
		if (res == STATE_OK)
		{
			boost::recursive_mutex::scoped_lock lock(mutex_.value);
			info_.emplace();
			bbs_->getInfo(*info_);
		}
		return res;
	}
	return STATE_ERROR_INVALID;
}

int BBSManager::loadSubject(const std::wstring& userAgent, int timeout)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.bbs);
	if (bbs_ && *bbs_)
	{
		int res = bbs_->loadSubject(userAgent, timeout);

		if (res == STATE_OK)
		{
			boost::recursive_mutex::scoped_lock lock(mutex_.value);
			info_.emplace();
			bbs_->getInfo(*info_);

			subject_.emplace();
			bbs_->getSubject(*subject_);
		}
		return res;
	}
	return STATE_ERROR_INVALID;
}

int BBSManager::loadDat(const std::wstring& userAgent, int timeout)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.bbs);
	if (bbs_ && *bbs_)
	{
		int res = bbs_->loadDat(userAgent, timeout);

		if (res == STATE_OK)
		{
			boost::recursive_mutex::scoped_lock lock(mutex_.value);
			info_.emplace();
			bbs_->getInfo(*info_);

			bool equal = pos_ && dat_ ? pos_.get() == (int)dat_->size() : false;
			bool init = !pos_ || !dat_ || equal;

			dat_.emplace();
			bbs_->getDat(*dat_);

			if (init)
			{
				if (dat_)
				{
					pos_ = (int)dat_->size();
				}
				else
				{
					pos_.reset();
				}
			}
		}
		return res;
	}
	return STATE_ERROR_INVALID;
}


bool BBSManager::getSubject(std::vector<ThreadInfo>& data)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!subject_) { return false; }
	data = *subject_;
	return true;
}

bool BBSManager::getDat(std::vector<ResInfo>& data, int diff, int len)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!dat_) { return false; }

	if (len < 0)
	{
		data = *dat_;
		return true;
	}

	int size = (int)dat_->size();
	if (size <= 0) { return false; }

	int org = 0;
	if (pos_) { org = *pos_; }
	if (org <= 0) { return false; }

	int pos = size;
	if (diff != INT_MAX)
	{
		len = min(size, len);
		pos = max(min(org + diff, size), len);
	}

	if (pos <= 0) { return false; }
	if (diff != 0 && org == pos) { return false; }

	pos_ = pos;

	//---------------------------------------------

	auto first = dat_->begin();

	if (diff <= 0)
	{
		first += pos;
	}
	else
	{
		first += (pos - 1);
		for (; first != dat_->end(); ++first, ++pos)
		{
			if (!first->del) { break; }
		}
		first++;
	}

	int i = 1;
	data.clear();
	std::deque<ResInfo> d;
	bool flag = true;
	for (std::vector<ResInfo>::const_reverse_iterator it(first); it != dat_->rend(); ++it)
	{
		if (i > len) { break; }
		int now = pos--;
		if (it->del) { continue; }

		if (flag) { flag = false; pos_ = now; }

		++i;
		d.push_front(*it);
	}
	data.assign(d.begin(), d.end());
	return true;
}

bool BBSManager::getPos(int& pos)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!pos_) { return false; }
	pos = *pos_;
	return true;
}

int BBSManager::post(const std::wstring& userAgent, int timeout, const std::wstring& name, const std::wstring& mail, const std::wstring& message,
	int& code, std::wstring& error)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.bbs);
	if (bbs_ && *bbs_)
	{
		return bbs_->post(userAgent, timeout, name, mail, message, code, error);
	}
	return STATE_ERROR_INVALID;
}

//---------------------------------------------------------------------------------

bool BBSManager::getURL(std::wstring& url)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	if (info_->thread.key.empty())
	{
		url = info_->board.url;
	}
	else
	{
		url = info_->thread.url;
	}
	return true;
}

bool BBSManager::getBoardURL(std::wstring& url)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	url = info_->board.url;
	return true;
}

bool BBSManager::getThreadURL(std::wstring& url)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	url = info_->thread.url;
	return true;
}

bool BBSManager::getBoardTitle(std::wstring& title)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	title = info_->board.title;
	return true;
}

bool BBSManager::getThreadTitle(std::wstring& title)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	title = info_->thread.title;
	return true;
}

bool BBSManager::getNoName(std::wstring& noname)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	noname = info_->board.noname;
	return true;
}

bool BBSManager::getCount(int& count)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	count = info_->thread.count;
	return true;
}

bool BBSManager::getKey(std::wstring& key)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	key = info_->thread.key;
	return true;
}

bool BBSManager::hasKey()
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	return !info_->thread.key.empty();
}

bool BBSManager::getStop(int& stop)
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	stop = info_->thread.stop;
	return true;
}

bool BBSManager::isStop()
{
	boost::recursive_mutex::scoped_lock lock(mutex_.value);
	if (!info_) { return false; }

	return info_->thread.count >= info_->thread.stop;
}


//---------------------------------------------------------------------------------

bool BBSManager::match(const std::wstring& str, const std::wstring& rx)
{
	using namespace boost::xpressive;
	
	try {
		wsmatch what;
		if (regex_match(str, what, wsregex::compile(rx)))
		{
			return true;
		}
	} catch (...) {}
	return false;
}

}; // namespace bbs
