#pragma once

#define WM_GRAPH_NOTIFY	(WM_APP + 20)

#include <boost/thread.hpp>
#include <boost/functional.hpp>

#include "GraphManager.h"

namespace dsp {

// Otʒm
typedef enum {
	GRAPH_NOTIFY_START, // ĐJn
	GRAPH_NOTIFY_END, // ĐI
	GRAPH_NOTIFY_RESTART, // čĐ
	GRAPH_NOTIFY_CONNECT, // `lڑI
	GRAPH_NOTIFY_RETRY_END, // `lڑgCI
} GRAPH_NOTIFY;

// I[fBI
struct AudioControl {
	AudioControl()
		: volume(50)
		, balance(0)
		, mute(false)
	{}

	int volume; // 
	int balance; // ʃoX
	bool mute; // ~[g
};

// rfI
struct VideoControl {
	VideoControl()
		: procamp()
		, nominal(dsp::NOMINAL_RANGE_0_255)
		, resizer(d3dutl::RESIZER_DEFAULT)
	{}

	virtual ~VideoControl() {}

	ProcAmpValue procamp; // F␳
	int nominal; // o͔͈
	int resizer; // TCY
};

// lbg[N
struct NetworkControl {
	// gC(PeerCast)
	struct Retry {
		Retry()
			: timeout(10000)
			, count(5)
			, wait(5)
			, stop(true)
		{}

		int timeout; // ^CAEg
		int count; // gC
		int wait; // gCԊu
		bool stop; // gCIɃ`lؒf
	};

	// č\z(PeerCast)
	struct Rebuild {
		Rebuild()
			: enable(true)
			, count(5)
		{}

		bool enable; // w肵č\z񐔂ƂɃ`lĐڑ
		int count; // č\z
	};

	// URL Buffer Reader
	struct Reader {
		Reader()
			: userAgent(L"PCRPlayer/1.00")
			, timeout(10000)
		{}

		std::wstring userAgent; // userAgent
		int timeout; // ^CAEg(ms)
	};

public:
	Retry retry; // gC(PeerCast)
	Rebuild rebuild; // č\z(PeerCast)
	Reader reader; // URL Buffer Reader
};

// ^Cvʐݒ
struct TypeControl {
	// FLV
	struct FLV {
		FLV()
			: loop(true)
			, init(false)
			, offset(1500)
			, delay(false)
			, buffer(3000)
			, timestamp(true)
		{}

		bool loop; // [vo`lĐڑ
		bool init; // obt@L
		int offset; // obt@(ms)
		bool delay; // x␳L
		int buffer; // obt@(ms)
		bool timestamp; // ĐԂ^CX^vŕ\
	};

	FLV flv; // FLV
};

// fR[_ݒ
struct DecoderControl {
	DecoderControl()
		: dxva2(true)
	{}

	bool dxva2; // DXVA2Lɂ
};


struct BuildConfig {
	BuildConfig() {}

	AudioControl audio; // I[fBI
	VideoControl video; // rfI
	NetworkControl network; // lbg[N
	TypeControl type; // ^Cvʐݒ
	DecoderControl decoder; // fR[_ݒ
	DecoderIDSet decoderSet; // fR[_Zbg
};

struct BuildStatus {
	BuildStatus()
		: hwnd(NULL)
		, msg(0)
		, path()
	{}

	HWND hwnd;
	UINT msg;
	std::wstring path;
};

class BuildStatusMutex {
	boost::recursive_mutex mutex_;
	BuildStatus status_;

public:
	void set(HWND hwnd, UINT msg, const std::wstring& path)
	{
		boost::recursive_mutex::scoped_lock lock(mutex_);
		status_.hwnd = hwnd;
		status_.msg = msg;
		status_.path = path;
	}

	BuildStatus get()
	{
		boost::recursive_mutex::scoped_lock lock(mutex_);
		return status_;
	}
};

// Ot}`Xbh
class IBaseBuilder : protected utl::Logger {
protected:
	utl::SyncOptional<std::wstring>& text_; // Xe[^XeLXg

protected:
	BuildStatusMutex status_;

	boost::thread thread_;
	boost::thread run_;
	boost::recursive_mutex mutex_;
	GraphManager graph_;

	boost::recursive_mutex mxcfg_;
	BuildConfig cfg_;

protected:
	// obt@ʒm
	struct Buffer {
		Buffer() : init(false), enable(false), count(0) {}

		bool init;
		bool enable;
		int count;
	};

	// ĐڑJE^
	struct Bump {
		Bump() : count(0), time(0) {}
		int count;
		DWORD time;

		bool update(DWORD newtime)
		{
			if (time == 0 || newtime - time > 10000)
			{
				time = newtime;
				return true;
			}
			return false;
		}
	};

	Buffer buffer_;
	Bump bump_;

protected: // ݒl
	int volume_;
	int balance_;

public:
	IBaseBuilder(utl::SyncLog& log, utl::SyncOptional<std::wstring>& text);
	virtual ~IBaseBuilder();

	void mainloop(HWND hwnd, UINT msg, const std::wstring& path);
	bool build(HWND hwnd, UINT msg, const std::wstring& path);

	void execute(HWND hwnd, UINT msg, const std::wstring& path, const BuildConfig& cfg);
	bool terminate();

	bool getState(long timeout, long& state);
	bool isRunning(long timeout = 0);
	bool hasAudio();
	bool hasVideo();
	bool isAudioRunning(long timeout = 0);
	bool isVideoRunning(long timeout = 0);

	bool run();
	virtual bool pause();
	virtual bool togglePause();
	bool stop();
	bool abort();

	bool getEvent(long& code, LONG_PTR& param1, LONG_PTR& param2);
	bool repaint(HDC hdc);
	bool setVideoPosition(RECT& dst);
	bool getNativeVideoSize(SIZE& size);
	bool saveCurrentImage(const std::wstring& path, int format, int jpegQuality, UINT width, UINT height, bool fit);

	bool getStatusText(std::wstring& text);

	bool getSampleTime(LONGLONG& time);
	bool getPositionDiff(LONGLONG& diff);
	bool getDuration(LONGLONG& duration);
	bool getCurrentPosition(LONGLONG& pos);
	bool setPositions(LONGLONG current, LONGLONG stop);
	bool getPositions(LONGLONG& current, LONGLONG& stop);
	virtual bool seek(LONGLONG current);

	bool getProgress(int& percent);
	bool getCurrentBitrate(int& bitrate);
	bool getFramerate(int& rate);
	bool getCurrentFramerate(int& rate);

	void getBalance(int& balance);
	void getVolume(int& volume);
	void getMute(bool& mute);

	bool setBalance(int balance);
	bool setVolume(int volume);
	bool setMute(bool mute);

	bool getProcAmpRange(ProcAmpRangeGroup& group);
	bool setProcAmp(const ProcAmpValue& value);
	bool getProcAmp(ProcAmpValue& value);

	bool setNominalRange(int value);

	bool setResizer(int type);

	bool setNetwork(const NetworkControl& network);
	bool setType(const TypeControl& type);

	bool setDecoderSet(const DecoderIDSet& decoderSet);

	bool getFilterProperty(std::vector<FilterProperty>& prop);
	bool showFilterProperty(int index, HWND parent, HINSTANCE hi, DWORD id);

	bool setInit(bool init);
	bool setOffset(int offset);
	bool setDelay(bool delay);
	bool setBuffer(int buffer);

private:
	void runThread(HWND hwnd, UINT msg);

protected:
	virtual bool render(HWND hwnd, UINT msg, const std::wstring& path) = 0;
	virtual bool onAbort(HWND hwnd, UINT msg, const std::wstring& path, LONG_PTR p1, LONG_PTR p2) { return true; }
	virtual bool onComplete(HWND hwnd, UINT msg, const std::wstring& path, LONG_PTR p1, LONG_PTR p2) { return true; }
	virtual bool onBufferEnter(HWND hwnd, UINT msg, const std::wstring& path, LONG_PTR p1, LONG_PTR p2) { return true; }
	virtual bool onBufferExit(HWND hwnd, UINT msg, const std::wstring& path, LONG_PTR p1, LONG_PTR p2) { return true; }
	virtual bool onEnterLoop(HWND hwnd, UINT msg, const std::wstring& path) { return true; }
	virtual bool onLoop(HWND hwnd, UINT msg, const std::wstring& path) { return true; }
	virtual bool onExitLoop(HWND hwnd, UINT msg, const std::wstring& path) { return true; }
	virtual bool onRebuild(HWND hwnd, UINT msg, const std::wstring& path) { return true; }
};

} // namespace dsp

